# -*- coding: utf-8 -*-

from django.conf import settings

import tripleStore.oauth.settings as osettings

def _sign(v):
	from django.core.signing import Signer
	signer = Signer()
	value = signer.sign(v)
	return value

def _unsign(v):
	from django.core.signing import Signer
	signer = Signer()
	original = signer.unsign(v)
	return original

def get_client_ip(request):
	x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
	if x_forwarded_for:
		ip = x_forwarded_for.split(',')[-1].strip()
	else:
		ip = request.META.get('REMOTE_ADDR')
	return ip

def verifyCaptcha(request):
	remoteip = get_client_ip(request)
	if hasattr(settings, 'RECAPTCHA_PRIVATE_KEY'):
		privatekey = settings.RECAPTCHA_PRIVATE_KEY
	else:
		return None

	response = request.POST.get('g-recaptcha-response', '')
	import urllib2
	URL = "https://www.google.com/recaptcha/api/siteverify?secret="
	URL = URL + privatekey
	URL = URL + "&response=" + response
	URL = URL + "&remoteip=" + remoteip
	data = urllib2.urlopen(URL)
	rs = data.read()
	import json
	return json.loads(rs)['success']


	response = request.POST.get('recaptcha_response_field','')
	challenge = request.POST.get('recaptcha_challenge_field','')

	import urllib
	post_data = [('remoteip',remoteip),('privatekey',privatekey),('challenge',challenge),('response', response)]
	result = urllib.urlopen('http://www.google.com/recaptcha/api/verify', urllib.urlencode(post_data))
	content = result.read()
	# return content
	content = content.split("\n")
	if content[0]=="true":
		return True
	return False

def passwordStrength(p):
	if p.isupper() and p.islower():
		return "your password is too weak, try adding some numbers"
	if len(p)<6:
		return "your password must be longer than 5 characters"
	if p.isdigit():
		return "your password contains only digits"
		
	return True

def validEmail(m):
	from validate_email import validate_email
	return validate_email(m)

def sendPasswordReminder(email):
	import tripleStore.VirtuosoHelper as VH
	from django.core.mail import send_mail
	from uuid import uuid4

	if not validEmail(email):
		return "Not a valid email"
	
	if VH.emailExists(email)==0:
		return "Email not registered"

	code = str(uuid4())
	url = osettings.settings['carreSecureURL'] + "/devices/accounts/recover?code=" + code
	subject = osettings.settings['carreSecureURL'] + "/" + email
	predicate = osettings.settings['carrePasswordCodePredicate']
	object = "\"" + code + "\"^^xsd:string"
	VH.insertUniqueTriple("__internals__", subject, predicate, object, "dba", "DBA_PASSWORD_HERE")
	username = VH.getUserByEmail(email)

	message = "Your username is " + username + "\n"
	message = message + "Click on the following link to reset your password\n\n" + url 
	send_mail('CARRE devices password reset', message, 'noreply@carre.kmi.open.ac.uk', [email], fail_silently=False)
	return 1

def fetchGoogleCalendar(token, calendarId='primary'):
	from oauth2client.client import AccessTokenCredentials
	clientid = osettings.settings['google-clientid']
	credentials = AccessTokenCredentials(token, "agent")
	import httplib2
	http = httplib2.Http()
	http = credentials.authorize(http)
	from apiclient.discovery import build
	service = build(serviceName='calendar', version='v3', http=http,developerKey=clientid)
	page_token = None
	rs = []
	while True:
		events = service.events().list(calendarId=calendarId, pageToken=page_token).execute()
		for event in events['items']:
			rs.append(event)
		page_token = events.get('nextPageToken')
		if not page_token:
			break

	return rs

def storeIhealth(username, password, tokens):
	import tripleStore.VirtuosoHelper as VH
	from uuid import uuid4
	import datetime
	
	graphName = "<" + osettings.settings['carreSecureURL'] + "/users/" + username  + ">"
	connectionId = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + str(uuid4()) + ">"
	
	#user has a new connection
	subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + ">"
	predicate = "<" + osettings.settings['carreConnectionPredicate'] + ">"
	object = connectionId
	triples =[]
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)

	#connection has ihealth manufacturer
	subject = connectionId
	predicate = "<" + osettings.settings['carreManufacturerPredicate'] + ">"
	object = "<" + osettings.settings['carreManufacturerIRI'] + "/ihealth>"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)

	#1. connection has credentials (access token)
	subject = connectionId
	predicate = "<" + osettings.settings['carreAccessTokenPredicate'] + ">"
	object = "\"" + tokens['AccessToken'] + "\"^^xsd:string"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)
	
	#2. connection has credentials (expires)
	subject = connectionId
	predicate = "<" + osettings.settings['carreExpiryPredicate'] + ">"
	object = "\"" + str(tokens['Expires']) + "\"^^xsd:int"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)
	

	#2.5 connection expires
	now = datetime.datetime.today()
	e = now + datetime.timedelta(0,tokens['Expires']/1000)
	subject = connectionId#osettings.settings['carreManufacturerIRI'] + "/ihealth"
	subject = subject[1:]
	subject = subject[:-1]
	predicate = osettings.settings['carreExpiryDatePredicate']
	object = "\"" + e.strftime("%Y-%m-%dT%H:%M:%SZ") + "\"^^xsd:datetime"
	if VH.tripleExists(username, subject, predicate, username, password) == 0:
		rs = VH.insertUniqueTriple(username,subject, predicate, object, username, password)


	#3. connection has credentials (RefreshToken)
	subject = connectionId
	predicate = "<" + osettings.settings['carreRefreshTokenPredicate'] + ">"
	object = "\"" + tokens['RefreshToken'] + "\"^^xsd:string"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)
	
	#4. connection has credentials (UserID)
	subject = connectionId
	predicate = "<" + osettings.settings['carreUserIdPredicate'] + ">"
	object = "\"" + tokens['UserID'] + "\"^^xsd:string"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	rs = VH._insertTriplesToGraph(graphName, triples, username, password)

	#store carretime
	triples = []
	subject = osettings.settings['carreManufacturerIRI'] + "/ihealth"
	predicate = osettings.settings['carreLastMeasuredPredicate']
	object = "\"" + osettings.settings['carretime'] + "\"^^xsd:date"
	rs = VH.insertUniqueTriple(username,subject, predicate, object, username, password)

	#store first connected
	subject = osettings.settings['carreManufacturerIRI'] + "/ihealth"
	predicate = osettings.settings['carreFirstConnectedPredicate']
	object = "\"" + datetime.date.today().strftime("%Y-%m-%d") + "\"^^xsd:date"
	if VH.tripleExists(username, subject, predicate, username, password) == 0:
		rs = VH.insertUniqueTriple(username,subject, predicate, object, username, password)

	return rs

def storeConnectionCallback(userId=None, connectionId=None, access_token=None, access_token_secret=None, refresh_token=None, token_expiry=None, expiry=None, manufacturer=None, username=None, password=None):
	from uuid import uuid4
	import datetime
	import VirtuosoHelper as VH

	if connectionId is not None:
		connectionId = str(uuid4())
	
	graphName = "<" + osettings.settings['carreSecureURL'] + "/users/" + username  + ">"

	subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
	predicate = "<" + osettings.settings['carreManufacturerPredicate'] + ">"
	object = "<" + osettings.settings['carreManufacturerIRI'] + "/"+manufacturer+">"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)

	# userId
	if userId is not None:
		subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + ">"
		predicate = "<" + osettings.settings['carreConnectionPredicate'] + ">"
		object = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
		triples =[]
		triples.append(subject + " " + predicate + " " + object)
		VH._insertTriplesToGraph(graphName, triples, username, password)

	# access_token
	if access_token is not None:
		subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
		predicate = "<" + osettings.settings['carreAccessTokenPredicate'] + ">"
		triples =[]
		triples.append(subject + " " + predicate + " " + access_token)
		VH._insertTriplesToGraph(graphName, triples, username, password)

	# access_token_secret
	if access_token_secret is not None:
		subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
		predicate = "<" + osettings.settings['carreRequestTokenSecretPredicate'] + ">"
		object = "\"" + access_token_secret + "\"^^xsd:string"
		triples = []
		triples.append(subject + " " + predicate + " " + object)
		VH._insertTriplesToGraph(graphName, triples, username, password)

	# refresh_token
	if refresh_token is not None:
		subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
		predicate = "<" + osettings.settings['carreRefreshTokenPredicate'] + ">"
		triples =[]
		triples.append(subject + " " + predicate + " " + refresh_token)
		VH._insertTriplesToGraph(graphName, triples, username, password)

	# token_expiry
	if token_expiry is not None:
		subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
		predicate = "<" + osettings.settings['carreExpiryPredicate'] + ">"
		object = "\"" + token_expiry + "\"^^xsd:int"
		triples = []
		triples.append(subject + " " + predicate + " " + object)
		VH._insertTriplesToGraph(graphName, triples, username, password)

	# expiry
	if token_expiry is not None:
		now = datetime.datetime.now()
		expiry = now + datetime.timedelta(seconds=token_expiry)
		subject = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + "/" + osettings.settings['carreConnectionIRI'] + "/" + connectionId + ">"
		subject = subject[1:]
		subject = subject[:-1]
		predicate = osettings.settings['carreExpiryDatePredicate']
		object = "\"" + expiry.strftime("%Y-%m-%dT%H:%M:%SZ") + "\"^^xsd:datetime"
		VH.insertUniqueTriple(username,subject, predicate, object, username, password)

	# store first connected
	subject = osettings.settings['carreManufacturerIRI'] + "/" + manufacturer
	predicate = osettings.settings['carreFirstConnectedPredicate']
	object = "\"" + datetime.date.today().strftime("%Y-%m-%d") + "\"^^xsd:date"
	if VH.tripleExists(username, subject, predicate, username, password) == 0:
		VH.insertUniqueTriple(username,subject, predicate, object, username, password)

	#store carretime
	triples = []
	subject = osettings.settings['carreManufacturerIRI'] + "/" + manufacturer
	predicate = osettings.settings['carreLastMeasuredPredicate']
	if manufacturer=="moves":
		object = "\"" + (datetime.datetime.now() + datetime.timedelta(-31)).strftime("%Y-%m-%d") + "\"^^xsd:date"
	else:
		object = "\"" + osettings.settings['carretime'] + "\"^^xsd:date"
	VH.insertUniqueTriple(username,subject, predicate, object, username, password)


	return

def storeGoogleCalendar(username, password, connectionId, events, calendarId='primary'):
	import tripleStore.VirtuosoHelper as VH
	from uuid import uuid4

	#calendarId
	cid = str(uuid4())
	graphName = "<" + osettings.settings['carreSecureURL'] + "/users/" + username  + ">"
	subject = connectionId
	predicate = "<" + osettings.settings['carreCalendarIdPredicate'] + ">" 
	object = "<" + osettings.settings['carreSecureURL'] + "/users/" + username  + "/" + osettings.settings['carreCalendarIRI'] + "/" + cid + ">"
	triples = []
	triples.append(subject + " " + predicate + " " + object)
	VH._insertTriplesToGraph(graphName, triples, username, password)

	triples = []
	for event in events:
		wanted_keys = ['id', 'status', 'description', 'created', 'iCalUID', 'htmlLink', 'summary', 'location']
		e = dict([(i, event[i]) for i in wanted_keys if i in event])
		if 'start' in event.keys():
			if 'dateTime' in event['start'].keys():
				e['startTime'] = event['start']['dateTime']
		if 'end' in event.keys():
			if 'dateTime' in event['end'].keys():
				e['endTime'] = event['end']['dateTime']
		if 'id' in e.keys():
			e["<" + osettings.settings['carreCalendarEventIdPredicate'] +">"] = e.pop('id')
		if 'status' in e.keys():
			e["<" + osettings.settings['carreCalendarEventStatusPredicate'] + ">"] = "\"\"\"" + e.pop('status') + "\"\"\""	
		if 'description' in e.keys():
			e["<" + osettings.settings['carreCalendarEventDescriptionPredicate'] + ">"] = "\"\"\"" + e.pop('description') + "\"\"\""
		if 'created' in e.keys():
			e["<" + osettings.settings['carreCalendarEventCreationDatePredicate'] + ">"] = "\""+ e.pop('created')	+ "\"^^xsd:date"
		if 'iCalUID' in e.keys():
			e["<" + osettings.settings['carreCalendarEventICalUIDPredicate'] + ">"] = "\"\"\"" + e.pop('iCalUID')+ "\"\"\""
		if 'htmlLink' in e.keys():
			e["<" + osettings.settings['carreCalendarEventHtmlLinkPredicate'] + ">"] = "\"\"\""+e.pop('htmlLink')+ "\"\"\""
		if 'summary' in e.keys():
			e["<" + osettings.settings['carreCalendarEventSummaryPredicate'] + ">"] = "\"\"\""+e.pop('summary')+ "\"\"\""
		if 'location' in e.keys():
			e["<" + osettings.settings['carreCalendarEventLocationPredicate'] + ">"] = "\"\"\""+ e.pop('location')+ "\"\"\""
		if 'startTime' in e.keys():
			e["<" + osettings.settings['carreCalendarEventStartTimePredicate'] + ">"] = "\"" +e.pop('startTime') + "\"^^xsd:date"	
		if 'endTime' in e.keys():
			e["<" + osettings.settings['carreCalendarEventEndTimePredicate'] + ">"] = "\"" + e.pop('endTime') + "\"^^xsd:date"

		triples.extend(DictToTriples(e,identifier="<" + osettings.settings['carreCalendarEventIdPredicate'] + ">"))
	
	graphName = "<" + osettings.settings['carreSecureURL'] + "/users/" + username + ">"
	rs = VH._insertTriplesToGraph(graphName, triples, username, password)
	return rs
	if rs>0:
		return str(len(triples))
	return rs

def DataFrameToTriples(df, identifier='id', prefix=osettings.settings['carreBaseURL']):
	import pandas as pd
	import numpy as np

	if not isinstance(df, pd.core.frame.DataFrame):
		return ""

	if identifier not in df.columns:
		return ""

	rs = []
	try:
		d = "<" + osettings.settings['carreCalendarEventCreationDatePredicate'] + ">"
		df[d] = df[d].astype(np.datetime64)
	except Exception, e:
		df[d] = pd.NaT
	valueId = df[identifier].tolist()[0]
	valueId = str(valueId)
	df = df.fillna("")
	for column in df.columns:
		v = df[column].tolist()[0]
		if isinstance(v, int):
			v = str(v)
		if isinstance(v, np.int64):
			v = str(v)
		if pd.isnull(v):
			pass
		#v = v.replace('"','\\"')
		try:
			if column==identifier:
				rs.append("<" + prefix +  valueId + "> " + column + " \"" + v + "\"")
			else:
				rs.append("<" + prefix +  valueId + "> " + column + " " + v + "")
		except Exception, e:
			pass
	
	return rs

def DictToTriples(d, identifier='id', prefix=osettings.settings['carreBaseURL']):
	import pandas as pd
	import numpy as np
	df = pd.DataFrame(d, index=[0])
	return DataFrameToTriples(df, identifier=identifier, prefix=prefix)

def connectionExists(provider, username, password):
	import tripleStore.VirtuosoHelper as VH
	sql = "SPARQL SELECT count(?subject) as ?cnt FROM <" + osettings.settings['carreSecureURL'] + "/users/" + username + "> WHERE { ?subject <" + osettings.settings['carreManufacturerPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + ">}"
	data = VH._fetchTriples(sql, username, password)
	cnt = data.cnt.tolist()[0]
	return cnt

def connectionRemove(provider, username, password):
	import tripleStore.VirtuosoHelper as VH
	sql = "SPARQL SELECT ?connection FROM <" + osettings.settings['carreSecureURL'] + "/users/" + username + "> WHERE { ?connection <" + osettings.settings['carreManufacturerPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + "> }"
	data = VH._fetchTriples(sql, username, password)
	for c in data.connection:
		sql = "SPARQL WITH <" + osettings.settings['carreSecureURL'] + "/users/" + username + ">DELETE { ?connection ?property ?value }WHERE{ ?connection ?property ?value ; <" + osettings.settings['carreManufacturerPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + ">}"
		VH._execSQL(sql,username, password)
		sql = "SPARQL WITH <" + osettings.settings['carreSecureURL'] + "/users/" + username + ">DELETE { ?person ?property ?connection }WHERE{?person ?property ?connection . ?person <" + osettings.settings['carreConnectionPredicate'] + "> <" + c + ">}"
		VH._execSQL(sql,username, password)


	sql = "SPARQL SELECT ?connection FROM <" + osettings.settings['carreSecureURL'] + "/users/" + username + "> WHERE { ?connection <" + osettings.settings['carreIsMeasuredByPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + "> }"
	data = VH._fetchTriples(sql, username, password)
	for c in data.connection:
		sql = "SPARQL WITH <" + osettings.settings['carreSecureURL'] + "/users/" + username + ">DELETE { ?connection ?property ?value }WHERE{ ?connection ?property ?value ; <" + osettings.settings['carreIsMeasuredByPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + ">}"
		VH._execSQL(sql,username, password)

	sql = "SPARQL WITH <" + osettings.settings['carreSecureURL'] + "/users/" + username + ">DELETE { <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + "> ?property ?value } WHERE{ ?connection ?property ?value }"
	VH._execSQL(sql,username, password)

	return 1

def probeService(username, password, provider):
	import tripleStore.VirtuosoHelper as VH	
	provider = provider.lower()
	if provider not in ["fitbit", "withings", "ihealth", "google", "virtuoso", "misfit", "googlefit", "medisana", "moves"]:
		return 0
	sql = "SPARQL SELECT count(*) as ?cnt FROM <" + osettings.settings['carreSecureURL'] + "/users/" + username + "> WHERE { ?connection <" + osettings.settings['carreManufacturerPredicate'] + "> <" + osettings.settings['carreManufacturerIRI'] + "/" + provider + "> . ?connection <" + osettings.settings['carreAccessTokenPredicate'] + "> ?value}"
	# print sql
	data = VH._fetchTriples(sql, username, password)
	if data is None:
		return -1
	rs = data.cnt.tolist()[0]
	return rs

